@@ -26,7 +26,8 @@ from xpra.server.proxy.proxy_instance_process import ProxyInstanceProcess
 from xpra.server.server_core import ServerCore
 from xpra.server.control_command import ArgsControlCommand, ControlError
 from xpra.child_reaper import getChildReaper
-from xpra.scripts.config import make_defaults_struct, PROXY_START_OVERRIDABLE_OPTIONS
+from xpra.scripts.parsing import parse_bool
+from xpra.scripts.config import make_defaults_struct, PROXY_START_OVERRIDABLE_OPTIONS, OPTION_TYPES
 from xpra.scripts.main import parse_display_name, connect_to, start_server_subprocess
 from xpra.make_thread import start_thread
 
@@ -34,7 +35,8 @@ from xpra.make_thread import start_thread
 PROXY_SOCKET_TIMEOUT = envfloat("XPRA_PROXY_SOCKET_TIMEOUT", "0.1")
 PROXY_WS_TIMEOUT = envfloat("XPRA_PROXY_WS_TIMEOUT", "1.0")
 assert PROXY_SOCKET_TIMEOUT>0, "invalid proxy socket timeout"
-CAN_STOP_PROXY = envbool("XPRA_CAN_STOP_PROXY", False)
+CAN_STOP_PROXY = envbool("XPRA_CAN_STOP_PROXY", getuid()!=0)
+STOP_PROXY_SOCKET_TYPES = os.environ.get("XPRA_STOP_PROXY_SOCKET_TYPES", "unix-domain,named-pipe").split(",")
 
 
 MAX_CONCURRENT_CONNECTIONS = 200
@@ -169,8 +171,28 @@ class ProxyServer(ServerCore):
             self.send_disconnect(proto, "invalid request")
             return
         if is_req("stop"):
+            #global kill switch:
             if not CAN_STOP_PROXY:
-                self.send_disconnect(proto, "cannot stop proxy server")
+                msg = "cannot stop proxy server"
+                log.warn("Warning: %s", msg)
+                self.send_disconnect(proto, msg)
+                return
+            #verify socket type (only local connections by default):
+            try:
+                socktype = proto._conn.get_info().get("type")
+            except:
+                socktype = "unknown"
+            if socktype not in STOP_PROXY_SOCKET_TYPES:
+                msg = "cannot stop proxy server from a '%s' connection" % socktype
+                log.warn("Warning: %s", msg)
+                log.warn(" only from: %s", csv(STOP_PROXY_SOCKET_TYPES))
+                self.send_disconnect(proto, msg)
+                return
+            #connection must be authenticated:
+            if not proto.authenticators:
+                msg = "cannot stop proxy server from unauthenticated connections"
+                log.warn("Warning: %s", msg)
+                self.send_disconnect(proto, msg)
                 return
             self._requests.add(proto)
             #send a hello back and the client should then send its "shutdown-server" packet
@@ -366,7 +388,7 @@ class ProxyServer(ServerCore):
                 client_conn.set_active(True)
                 process = ProxyInstanceProcess(uid, gid, env_options, session_options, self._socket_dir,
                                                self.video_encoders, self.csc_modules,
-                                               client_conn, client_state, cipher, encryption_key, server_conn, c, message_queue)
+                                               client_conn, disp_desc, client_state, cipher, encryption_key, server_conn, c, message_queue)
                 log("starting %s from pid=%s", process, os.getpid())
                 self.processes[process] = (display, message_queue)
                 process.start()
@@ -406,6 +428,20 @@ class ProxyServer(ServerCore):
                 log.warn("Warning: ignoring invalid start override")
                 log.warn(" %s=%s", k, v)
                 continue
+            try:
+                vt = OPTION_TYPES[k]
+                if vt==str:
+                    v = bytestostr(v)
+                elif vt==bool:
+                    v = parse_bool(k, v)
+                elif vt==int:
+                    v = int(v)
+                elif vt==list:
+                    v = list(bytestostr(x) for x in v)
+            except ValueError:
+                log("start_new_session: override option %s", k, exc_info=True)
+                log.warn("Warning: ignoring invalid value %s for %s (%s)", v, k, vt)
+                continue
             log("start override: %s=%s", k, v)
             if v is not None:
                 fn = k.replace("-", "_")
@@ -416,6 +452,9 @@ class ProxyServer(ServerCore):
         cwd = None
         if uid>0:
             cwd = get_home_for_uid(uid) or None
+            if not cwd or not os.path.exists(cwd):
+                import tempfile
+                cwd = tempfile.gettempdir()
         log("starting new server subprocess: options=%s", opts)
         log("env=%s", env)
         log("args=%s", args)
@@ -501,7 +540,7 @@ class ProxyServer(ServerCore):
 
 
     def get_info(self, proto, *_args):
-        info = ServerCore.get_info(self, proto)
+        info = ServerCore.get_minimal_server_info(self, proto)
         info.setdefault("server", {})["type"] = "Python/GLib/proxy"
         #only show more info if we have authenticated
         #as the user running the proxy server process:
